/* oled32_clock
 *  
 * A dátum és az idő kijelzése egy SSD1306 vezérlőjű,
 * 128x32 felbontású OLED kijelzőn 
 * Az időt és a dátumot egy előre beállított DS3231 RTC-ből
 * olvassuk ki. Azt RTC és a kijelző is I2C buszon kommunikál.
 * 
 * Kapcsolás:
 *    Arduino       DS3231 RTC     SSD1306 128x32
 *   ----------    ------------    ------------------------                             ~
 *  |       A4 |--| SDA    SDA |--| SDA   2020-FEB-20 THU  |
 *  |       A5 |--| SCL    SCL |--| SCL      _     _   _   |     
 *  |       5V |--| VCC    VCC |--| VCC   |   | * | | | |  |      
 *  |      GND |--| GND    GND |--| GND   |   | * |_| |_|  | 
 *   ----------    ------------    ------------------------
 */
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include "Fonts/FreeSerifBold12pt7b.h"

#define SCREEN_WIDTH 128        // OLED display width, in pixels
#define SCREEN_HEIGHT 32        // OLED display height, in pixels
#define dec1(x)     (x &0x0f)+48
#define dec10(x)    ((x>>4)&0x07)+48 

Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, -1);
#define RTC_ADDRESS 0x68        // A DS3231 RTC I2C címe
#define LCD_ADDRESS 0x3C        // Az SSD1306 vezérlő I2C címe

char timeDate[16];              // Ide töltjük a fogadott adatokat
char lasttime = 0;              // A legutóbbi másodperc érték

void setup() {
  display.begin(SSD1306_SWITCHCAPVCC, LCD_ADDRESS); 
}

void loop() {
  I2C_burstRead(RTC_ADDRESS,0,7,timeDate);   // Dátum és idő beolvasása
  if (timeDate[0] != lasttime) {
    lasttime = timeDate[0];
    displayDateTime();
  }
}

//--- A dátum és az idő beolvasása az RTC-ből --------
void I2C_burstRead(char saddr, char maddr, int size, char* data) {
  Wire.beginTransmission(saddr);             // Start feltétel, slave address küldés
  Wire.write(maddr);                         // Memóriacím kiküldése
  Wire.endTransmission();                    // Írás vége
  Wire.requestFrom(saddr, size);             // Restart és adatok beolvasása
  for (int i = 0; i < size; i++) {           // Az adatok eltárolása
    data[i] = Wire.read();                   // Az beolvasott adatok eltárolása
  }
}


//--- A dátum és az idő kiírása ------
void displayDateTime() {
  char daysOfTheWeek[8][3] = {"   ", "MON", "TUE", "WED", "THU", "FRI", "SAT", "SUN"};
  char sDate[15], sTime[10];
  //--- A dátum kiíratása YYYY-MM-DD DOW formátumban ---
  sDate[0] = '2';                                // Évszázad = 20xx
  sDate[1] = '0'; 
  sDate[2] = dec10(timeDate[6]);                 // Évtizedek 
  sDate[3] = dec1(timeDate[6]);                  // Évek  
  sDate[4] = '-'; 
  sDate[5] = dec10(timeDate[5]);                 // Tízhónapok  
  sDate[6] = dec1(timeDate[5]);                  // Hónapok 
  sDate[7] = '-';
  sDate[8] = dec10(timeDate[4]);                 // Tíznapok
  sDate[9] = dec1(timeDate[4]);                  // Napok
  sDate[10] = ' ';
  for(int i=0; i<3; i++) {
    sDate[11+i] = daysOfTheWeek[timeDate[3]][i];
  }
  sDate[14] = 0;
  //--- Az idő kiíratása HH:MM:SS formátumban ---
  sTime[0] = dec10(timeDate[2]);                 // Tízórák
  sTime[1] = dec1(timeDate[2]);                  // Órák
  sTime[2] = ':';
  sTime[3] = dec10(timeDate[1]);                 // Tízpercek
  sTime[4] = dec1(timeDate[1]);                  // Percek
  sTime[5] = ':';
  sTime[6] = dec10(timeDate[0]);                 // Tízmásodpercek
  sTime[7] = dec1(timeDate[0]);                  // másodpercek
  sTime[8] = 0; 
  //--- Kiírás a képernyőre -----------
  display.clearDisplay();                        // Képernyő törlés
  display.setFont();                             // 7x10 font
  display.setTextSize(1);
  display.setCursor(5,1);
  display.print(sDate);                          // dátum kiírása
  display.setFont(&FreeSerifBold12pt7b);         // Serif bold 12 pt
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(5,30);
  display.print(sTime);                          // idő kiírása
  display.display();
}
